@@ -270,6 +270,7 @@ static struct tables *find_table(const char *database, const char *orig_tablenam
 	}
 
 	if (database == NULL) {
+		AST_LIST_UNLOCK(&psql_tables);
 		return NULL;
 	}
 
@@ -417,6 +418,9 @@ static struct columns *find_column(struct tables *t, const char *colname)
 	return NULL;
 }
 
+#define IS_SQL_LIKE_CLAUSE(x) ((x) && ast_ends_with(x, " LIKE"))
+static char *ESCAPE_CLAUSE = " ESCAPE '\\'";
+
 static struct ast_variable *realtime_pgsql(const char *database, const char *tablename, const struct ast_variable *fields)
 {
 	RAII_VAR(PGresult *, result, NULL, PQclear);
@@ -426,6 +430,7 @@ static struct ast_variable *realtime_pgsql(const char *database, const char *tab
 	char *stringp;
 	char *chunk;
 	char *op;
+	char *escape = "";
 	const struct ast_variable *field = fields;
 	struct ast_variable *var = NULL, *prev = NULL;
 
@@ -440,6 +445,16 @@ static struct ast_variable *realtime_pgsql(const char *database, const char *tab
 		return NULL;
 	}
 
+	/*
+	 * Must connect to the server before anything else as ESCAPE_STRING()
+	 * uses pgsqlConn
+	 */
+	ast_mutex_lock(&pgsql_lock);
+	if (!pgsql_reconnect(database)) {
+		ast_mutex_unlock(&pgsql_lock);
+		return NULL;
+	}
+
 	/* Get the first parameter and first value in our list of passed paramater/value pairs */
 	if (!field) {
 		ast_log(LOG_WARNING,
@@ -448,38 +463,51 @@ static struct ast_variable *realtime_pgsql(const char *database, const char *tab
 			PQfinish(pgsqlConn);
 			pgsqlConn = NULL;
 		}
+		ast_mutex_unlock(&pgsql_lock);
 		return NULL;
 	}
 
 	/* Create the first part of the query using the first parameter/value pairs we just extracted
 	   If there is only 1 set, then we have our query. Otherwise, loop thru the list and concat */
-	op = strchr(field->name, ' ') ? "" : " =";
+	if (!strchr(field->name, ' ')) {
+		op = " =";
+	} else {
+		op = "";
+		if (IS_SQL_LIKE_CLAUSE(field->name)) {
+			escape = ESCAPE_CLAUSE;
+		}
+	}
 
 	ESCAPE_STRING(escapebuf, field->value);
 	if (pgresult) {
 		ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+		ast_mutex_unlock(&pgsql_lock);
 		return NULL;
 	}
 
-	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'", tablename, field->name, op, ast_str_buffer(escapebuf));
+	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'%s", tablename, field->name, op, ast_str_buffer(escapebuf), escape);
 	while ((field = field->next)) {
-		if (!strchr(field->name, ' '))
+		escape = "";
+		if (!strchr(field->name, ' ')) {
 			op = " =";
-		else
+		} else {
 			op = "";
+			if (IS_SQL_LIKE_CLAUSE(field->name)) {
+				escape = ESCAPE_CLAUSE;
+			}
+		}
 
 		ESCAPE_STRING(escapebuf, field->value);
 		if (pgresult) {
 			ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+			ast_mutex_unlock(&pgsql_lock);
 			return NULL;
 		}
 
-		ast_str_append(&sql, 0, " AND %s%s '%s'", field->name, op, ast_str_buffer(escapebuf));
+		ast_str_append(&sql, 0, " AND %s%s '%s'%s", field->name, op, ast_str_buffer(escapebuf), escape);
 	}
 
 	/* We now have our complete statement; Lets connect to the server and execute it. */
-	ast_mutex_lock(&pgsql_lock);
-
         if (pgsql_exec(database, tablename, ast_str_buffer(sql), &result) != 0) {
 		ast_mutex_unlock(&pgsql_lock);
 		return NULL;
@@ -540,6 +568,7 @@ static struct ast_config *realtime_multi_pgsql(const char *database, const char
 	char *stringp;
 	char *chunk;
 	char *op;
+	char *escape = "";
 	struct ast_variable *var = NULL;
 	struct ast_config *cfg = NULL;
 	struct ast_category *cat = NULL;
@@ -558,6 +587,16 @@ static struct ast_config *realtime_multi_pgsql(const char *database, const char
 	if (!(cfg = ast_config_new()))
 		return NULL;
 
+	/*
+	 * Must connect to the server before anything else as ESCAPE_STRING()
+	 * uses pgsqlConn
+	 */
+	ast_mutex_lock(&pgsql_lock);
+	if (!pgsql_reconnect(database)) {
+		ast_mutex_unlock(&pgsql_lock);
+		return NULL;
+	}
+
 	/* Get the first parameter and first value in our list of passed paramater/value pairs */
 	if (!field) {
 		ast_log(LOG_WARNING,
@@ -566,6 +605,7 @@ static struct ast_config *realtime_multi_pgsql(const char *database, const char
 			PQfinish(pgsqlConn);
 			pgsqlConn = NULL;
 		}
+		ast_mutex_unlock(&pgsql_lock);
 		ast_config_destroy(cfg);
 		return NULL;
 	}
@@ -578,43 +618,53 @@ static struct ast_config *realtime_multi_pgsql(const char *database, const char
 	/* Create the first part of the query using the first parameter/value pairs we just extracted
 	   If there is only 1 set, then we have our query. Otherwise, loop thru the list and concat */
 
-	if (!strchr(field->name, ' '))
+	if (!strchr(field->name, ' ')) {
 		op = " =";
-	else
+		escape = "";
+	} else {
 		op = "";
+		if (IS_SQL_LIKE_CLAUSE(field->name)) {
+			escape = ESCAPE_CLAUSE;
+		}
+	}
 
 	ESCAPE_STRING(escapebuf, field->value);
 	if (pgresult) {
 		ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+		ast_mutex_unlock(&pgsql_lock);
 		ast_config_destroy(cfg);
 		return NULL;
 	}
 
-	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'", table, field->name, op, ast_str_buffer(escapebuf));
+	ast_str_set(&sql, 0, "SELECT * FROM %s WHERE %s%s '%s'%s", table, field->name, op, ast_str_buffer(escapebuf), escape);
 	while ((field = field->next)) {
-		if (!strchr(field->name, ' '))
+		escape = "";
+		if (!strchr(field->name, ' ')) {
 			op = " =";
-		else
+			escape = "";
+		} else {
 			op = "";
+			if (IS_SQL_LIKE_CLAUSE(field->name)) {
+				escape = ESCAPE_CLAUSE;
+			}
+		}
 
 		ESCAPE_STRING(escapebuf, field->value);
 		if (pgresult) {
 			ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+			ast_mutex_unlock(&pgsql_lock);
 			ast_config_destroy(cfg);
 			return NULL;
 		}
 
-		ast_str_append(&sql, 0, " AND %s%s '%s'", field->name, op, ast_str_buffer(escapebuf));
+		ast_str_append(&sql, 0, " AND %s%s '%s'%s", field->name, op, ast_str_buffer(escapebuf), escape);
 	}
 
 	if (initfield) {
 		ast_str_append(&sql, 0, " ORDER BY %s", initfield);
 	}
 
-
 	/* We now have our complete statement; Lets connect to the server and execute it. */
-	ast_mutex_lock(&pgsql_lock);
-
 	if (pgsql_exec(database, table, ast_str_buffer(sql), &result) != 0) {
 		ast_mutex_unlock(&pgsql_lock);
 		ast_config_destroy(cfg);
@@ -655,8 +705,10 @@ static struct ast_config *realtime_multi_pgsql(const char *database, const char
 
 		for (rowIndex = 0; rowIndex < num_rows; rowIndex++) {
 			var = NULL;
-			if (!(cat = ast_category_new("","",99999)))
+			cat = ast_category_new_anonymous();
+			if (!cat) {
 				continue;
+			}
 			for (i = 0; i < numFields; i++) {
 				stringp = PQgetvalue(result, rowIndex, i);
 				while (stringp) {
@@ -709,6 +761,17 @@ static int update_pgsql(const char *database, const char *tablename, const char
 		return -1;
 	}
 
+	/*
+	 * Must connect to the server before anything else as ESCAPE_STRING()
+	 * uses pgsqlConn
+	 */
+	ast_mutex_lock(&pgsql_lock);
+	if (!pgsql_reconnect(database)) {
+		ast_mutex_unlock(&pgsql_lock);
+		release_table(table);
+		return -1;
+	}
+
 	/* Get the first parameter and first value in our list of passed paramater/value pairs */
 	if (!field) {
 		ast_log(LOG_WARNING,
@@ -717,6 +780,7 @@ static int update_pgsql(const char *database, const char *tablename, const char
 			PQfinish(pgsqlConn);
 			pgsqlConn = NULL;
 		}
+		ast_mutex_unlock(&pgsql_lock);
 		release_table(table);
 		return -1;
 	}
@@ -730,6 +794,7 @@ static int update_pgsql(const char *database, const char *tablename, const char
 
 	if (!column) {
 		ast_log(LOG_ERROR, "PostgreSQL RealTime: Updating on column '%s', but that column does not exist within the table '%s'!\n", field->name, tablename);
+		ast_mutex_unlock(&pgsql_lock);
 		release_table(table);
 		return -1;
 	}
@@ -740,6 +805,7 @@ static int update_pgsql(const char *database, const char *tablename, const char
 	ESCAPE_STRING(escapebuf, field->value);
 	if (pgresult) {
 		ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+		ast_mutex_unlock(&pgsql_lock);
 		release_table(table);
 		return -1;
 	}
@@ -754,6 +820,7 @@ static int update_pgsql(const char *database, const char *tablename, const char
 		ESCAPE_STRING(escapebuf, field->value);
 		if (pgresult) {
 			ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+			ast_mutex_unlock(&pgsql_lock);
 			release_table(table);
 			return -1;
 		}
@@ -765,6 +832,7 @@ static int update_pgsql(const char *database, const char *tablename, const char
 	ESCAPE_STRING(escapebuf, lookup);
 	if (pgresult) {
 		ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", lookup);
+		ast_mutex_unlock(&pgsql_lock);
 		return -1;
 	}
 
@@ -773,8 +841,6 @@ static int update_pgsql(const char *database, const char *tablename, const char
 	ast_debug(1, "PostgreSQL RealTime: Update SQL: %s\n", ast_str_buffer(sql));
 
 	/* We now have our complete statement; Lets connect to the server and execute it. */
-	ast_mutex_lock(&pgsql_lock);
-
 	if (pgsql_exec(database, tablename, ast_str_buffer(sql), &result) != 0) {
 		ast_mutex_unlock(&pgsql_lock);
 		return -1;
@@ -841,12 +907,24 @@ static int update2_pgsql(const char *database, const char *tablename, const stru
 		return -1;
 	}
 
+	/*
+	 * Must connect to the server before anything else as ESCAPE_STRING()
+	 * uses pgsqlConn
+	 */
+	ast_mutex_lock(&pgsql_lock);
+	if (!pgsql_reconnect(database)) {
+		ast_mutex_unlock(&pgsql_lock);
+		release_table(table);
+		return -1;
+	}
+
 	ast_str_set(&sql, 0, "UPDATE %s SET", tablename);
 	ast_str_set(&where, 0, " WHERE");
 
 	for (field = lookup_fields; field; field = field->next) {
 		if (!find_column(table, field->name)) {
 			ast_log(LOG_ERROR, "Attempted to update based on criteria column '%s' (%s@%s), but that column does not exist!\n", field->name, tablename, database);
+			ast_mutex_unlock(&pgsql_lock);
 			release_table(table);
 			return -1;
 		}
@@ -854,6 +932,7 @@ static int update2_pgsql(const char *database, const char *tablename, const stru
 		ESCAPE_STRING(escapebuf, field->value);
 		if (pgresult) {
 			ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+			ast_mutex_unlock(&pgsql_lock);
 			release_table(table);
 			return -1;
 		}
@@ -868,6 +947,7 @@ static int update2_pgsql(const char *database, const char *tablename, const stru
 			PQfinish(pgsqlConn);
 			pgsqlConn = NULL;
 		}
+		ast_mutex_unlock(&pgsql_lock);
 		release_table(table);
 		return -1;
 	}
@@ -884,6 +964,7 @@ static int update2_pgsql(const char *database, const char *tablename, const stru
 		ESCAPE_STRING(escapebuf, field->value);
 		if (pgresult) {
 			ast_log(LOG_ERROR, "PostgreSQL RealTime: detected invalid input: '%s'\n", field->value);
+			ast_mutex_unlock(&pgsql_lock);
 			release_table(table);
 			return -1;
 		}
@@ -897,7 +978,7 @@ static int update2_pgsql(const char *database, const char *tablename, const stru
 
 	ast_debug(1, "PostgreSQL RealTime: Update SQL: %s\n", ast_str_buffer(sql));
 
-	/* We now have our complete statement; connect to the server and execute it. */
+	/* We now have our complete statement; Lets connect to the server and execute it. */
         if (pgsql_exec(database, tablename, ast_str_buffer(sql), &result) != 0) {
 		ast_mutex_unlock(&pgsql_lock);
 	        return -1;
@@ -942,6 +1023,16 @@ static int store_pgsql(const char *database, const char *table, const struct ast
 		return -1;
 	}
 
+	/*
+	 * Must connect to the server before anything else as ESCAPE_STRING()
+	 * uses pgsqlConn
+	 */
+	ast_mutex_lock(&pgsql_lock);
+	if (!pgsql_reconnect(database)) {
+		ast_mutex_unlock(&pgsql_lock);
+		return -1;
+	}
+
 	/* Get the first parameter and first value in our list of passed paramater/value pairs */
 	if (!field) {
 		ast_log(LOG_WARNING,
@@ -950,12 +1041,6 @@ static int store_pgsql(const char *database, const char *table, const struct ast
 			PQfinish(pgsqlConn);
 			pgsqlConn = NULL;
 		}
-		return -1;
-	}
-
-	/* Must connect to the server before anything else, as the escape function requires the connection handle.. */
-	ast_mutex_lock(&pgsql_lock);
-	if (!pgsql_reconnect(database)) {
 		ast_mutex_unlock(&pgsql_lock);
 		return -1;
 	}
@@ -976,6 +1061,7 @@ static int store_pgsql(const char *database, const char *table, const struct ast
 
 	ast_debug(1, "PostgreSQL RealTime: Insert SQL: %s\n", ast_str_buffer(sql1));
 
+	/* We now have our complete statement; Lets connect to the server and execute it. */
         if (pgsql_exec(database, table, ast_str_buffer(sql1), &result) != 0) {
 		ast_mutex_unlock(&pgsql_lock);
 	        return -1;
@@ -1019,27 +1105,27 @@ static int destroy_pgsql(const char *database, const char *table, const char *ke
 		return -1;
 	}
 
-	/* Get the first parameter and first value in our list of passed paramater/value pairs */
-	/*newparam = va_arg(ap, const char *);
-	newval = va_arg(ap, const char *);
-	if (!newparam || !newval) {*/
-	if (ast_strlen_zero(keyfield) || ast_strlen_zero(lookup))  {
-		ast_log(LOG_WARNING,
-				"PostgreSQL RealTime: Realtime destroy requires at least 1 parameter and 1 value to search on.\n");
-		if (pgsqlConn) {
-			PQfinish(pgsqlConn);
-			pgsqlConn = NULL;
-		};
-		return -1;
-	}
-
-	/* Must connect to the server before anything else, as the escape function requires the connection handle.. */
+	/*
+	 * Must connect to the server before anything else as ESCAPE_STRING()
+	 * uses pgsqlConn
+	 */
 	ast_mutex_lock(&pgsql_lock);
 	if (!pgsql_reconnect(database)) {
 		ast_mutex_unlock(&pgsql_lock);
 		return -1;
 	}
 
+	/* Get the first parameter and first value in our list of passed paramater/value pairs */
+	if (ast_strlen_zero(keyfield) || ast_strlen_zero(lookup))  {
+		ast_log(LOG_WARNING,
+				"PostgreSQL RealTime: Realtime destroy requires at least 1 parameter and 1 value to search on.\n");
+		if (pgsqlConn) {
+			PQfinish(pgsqlConn);
+			pgsqlConn = NULL;
+		}
+		ast_mutex_unlock(&pgsql_lock);
+		return -1;
+	}
 
 	/* Create the first part of the query using the first parameter/value pairs we just extracted
 	   If there is only 1 set, then we have our query. Otherwise, loop thru the list and concat */
@@ -1055,10 +1141,11 @@ static int destroy_pgsql(const char *database, const char *table, const char *ke
 
 	ast_debug(1, "PostgreSQL RealTime: Delete SQL: %s\n", ast_str_buffer(sql));
 
-        if (pgsql_exec(database, table, ast_str_buffer(sql), &result) != 0) {
+	/* We now have our complete statement; Lets connect to the server and execute it. */
+	if (pgsql_exec(database, table, ast_str_buffer(sql), &result) != 0) {
 		ast_mutex_unlock(&pgsql_lock);
-	        return -1;
-        }
+		return -1;
+	}
 
 	numrows = atoi(PQcmdTuples(result));
 	ast_mutex_unlock(&pgsql_lock);
@@ -1136,9 +1223,10 @@ static struct ast_config *config_pgsql(const char *database, const char *table,
 			}
 
 			if (strcmp(last, field_category) || last_cat_metric != atoi(field_cat_metric)) {
-				cur_cat = ast_category_new(field_category, "", 99999);
-				if (!cur_cat)
+				cur_cat = ast_category_new_dynamic(field_category);
+				if (!cur_cat) {
 					break;
+				}
 				ast_copy_string(last, field_category, sizeof(last));
 				last_cat_metric = atoi(field_cat_metric);
 				ast_category_append(cfg, cur_cat);
@@ -1231,6 +1319,7 @@ static int require_pgsql(const char *database, const char *tablename, va_list ap
 		if (!column) {
 			if (requirements == RQ_WARN) {
 				ast_log(LOG_WARNING, "Table %s requires a column '%s' of size '%d', but no such column exists.\n", tablename, elm, size);
+				res = -1;
 			} else {
 				struct ast_str *sql = ast_str_create(100);
 				char fieldtype[15];
@@ -1270,7 +1359,8 @@ static int require_pgsql(const char *database, const char *tablename, va_list ap
 				ast_debug(1, "About to run ALTER query on table '%s' to add column '%s'\n", tablename, elm);
 
 			        if (pgsql_exec(database, tablename, ast_str_buffer(sql), &result) != 0) {
-						ast_mutex_unlock(&pgsql_lock);
+					ast_mutex_unlock(&pgsql_lock);
+					release_table(table);
 				        return -1;
 			        }
 
@@ -1394,6 +1484,7 @@ static int parse_config(int is_reload)
 
 	ast_mutex_lock(&pgsql_lock);
 
+	/* XXX: Why would we do this before we're ready to establish a new connection? */
 	if (pgsqlConn) {
 		PQfinish(pgsqlConn);
 		pgsqlConn = NULL;
@@ -1501,13 +1592,18 @@ static int pgsql_reconnect(const char *database)
 
 	/* mutex lock should have been locked before calling this function. */
 
-	if (pgsqlConn && PQstatus(pgsqlConn) != CONNECTION_OK) {
+	if (pgsqlConn) {
+		if (PQstatus(pgsqlConn) == CONNECTION_OK) {
+			/* We're good? */
+			return 1;
+		}
+
 		PQfinish(pgsqlConn);
 		pgsqlConn = NULL;
 	}
 
 	/* DB password can legitimately be 0-length */
-	if ((!pgsqlConn) && (!ast_strlen_zero(dbhost) || !ast_strlen_zero(dbsock)) && !ast_strlen_zero(dbuser) && !ast_strlen_zero(my_database)) {
+	if ((!ast_strlen_zero(dbhost) || !ast_strlen_zero(dbsock)) && !ast_strlen_zero(dbuser) && !ast_strlen_zero(my_database)) {
 		struct ast_str *conn_info = ast_str_create(128);
 
 		if (!conn_info) {
@@ -1605,7 +1701,7 @@ static char *handle_cli_realtime_pgsql_cache(struct ast_cli_entry *e, int cmd, s
 static char *handle_cli_realtime_pgsql_status(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	char status[256], credentials[100] = "";
-	int ctimesec = time(NULL) - connect_time;
+	int is_connected = 0, ctimesec = time(NULL) - connect_time;
 
 	switch (cmd) {
 	case CLI_INIT:
@@ -1621,7 +1717,11 @@ static char *handle_cli_realtime_pgsql_status(struct ast_cli_entry *e, int cmd,
 	if (a->argc != 4)
 		return CLI_SHOWUSAGE;
 
-	if (pgsqlConn && PQstatus(pgsqlConn) == CONNECTION_OK) {
+	ast_mutex_lock(&pgsql_lock);
+	is_connected = (pgsqlConn && PQstatus(pgsqlConn) == CONNECTION_OK);
+	ast_mutex_unlock(&pgsql_lock);
+
+	if (is_connected) {
 		if (!ast_strlen_zero(dbhost))
 			snprintf(status, sizeof(status), "Connected to %s@%s, port %d", dbname, dbhost, dbport);
 		else if (!ast_strlen_zero(dbsock))
