@@ -41,6 +41,7 @@
 
 /*** MODULEINFO
 	<depend>generic_odbc</depend>
+	<depend>res_odbc_transaction</depend>
 	<depend>ltdl</depend>
 	<support_level>core</support_level>
  ***/
@@ -522,7 +523,7 @@ static int load_odbc_config(void)
 						!strncasecmp(v->name, "share", 5) ||
 						!strcasecmp(v->name, "limit") ||
 						!strcasecmp(v->name, "idlecheck")) {
-					ast_log(LOG_WARNING, "The 'pooling', 'shared_connections', 'limit', and 'idlecheck' options are deprecated. Please see UPGRADE.txt for information\n");
+					ast_log(LOG_WARNING, "The 'pooling', 'shared_connections', 'limit', and 'idlecheck' options were replaced by 'max_connections'.  See res_odbc.conf.sample.\n");
 				} else if (!strcasecmp(v->name, "enabled")) {
 					enabled = ast_true(v->value);
 				} else if (!strcasecmp(v->name, "pre-connect")) {
