@@ -863,7 +863,7 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	ast_sorcery_apply_config(ast_sip_get_sorcery(), "asterisk-publication");
+	ast_sorcery_apply_config(ast_sip_get_sorcery(), "res_pjsip_publish_asterisk");
 	ast_sorcery_apply_default(ast_sip_get_sorcery(), "asterisk-publication", "config", "pjsip.conf,criteria=type=asterisk-publication");
 
 	if (ast_sorcery_object_register(ast_sip_get_sorcery(), "asterisk-publication", asterisk_publication_config_alloc, NULL, NULL)) {
